<?php

namespace MontonioPayment\Controller\Backend;

use Exception;
use Montonio\Clients\AbstractClient;
use Montonio\Clients\PaymentsClient;
use MontonioPayment\Services\ConfigService;
use MontonioPayment\Traits\LoggingTrait;
use Symfony\Component\HttpFoundation\Response;

class MontonioBackendTest extends \Shopware_Controllers_Backend_ExtJs
{
    use LoggingTrait;

    public function runAction()
    {
        /** @var ConfigService $configService */
        $configService = Shopware()->Container()->get(ConfigService::class);
        try {
            $client = new PaymentsClient($this->request->get('accessKey'), $this->request->get('secretKey'), AbstractClient::ENVIRONMENT_LIVE);
            $methods = $client->getPaymentMethods();
            if (empty($methods['DE'])) {
                if (!empty($methods['message'])) {
                    if ($methods['statusCode'] === 401) {
                        throw new Exception($configService->getBackendSnippet('credentialTestInvalid'));
                    } else {
                        throw new Exception($methods['message']);
                    }
                } else {
                    throw new Exception($configService->getBackendSnippet('credentialTestFailure'));
                }

            }
        } catch (Exception $exception) {
            $this->error($configService->getBackendSnippet('credentialTestFailure') . ': ' . $exception->getMessage(), ['request' => $this->request->getParams()]);
            $this->response->setStatusCode(Response::HTTP_EXPECTATION_FAILED);
            $this->View()->assign('response', $exception->getMessage());
            return;
        }
        $this->View()->assign('response', $configService->getBackendSnippet('credentialTestSuccess'));
    }
}
