<?php

namespace MontonioPayment;

use Enlight_Controller_Action;
use Enlight_Event_EventArgs;
use MontonioPayment\Components\CompilerPass\SmartyPluginCompilerPass;
use MontonioPayment\Services\ConfigService;
use MontonioPayment\Services\PaymentMethodService;
use Shopware\Components\Model\ModelManager;
use Shopware\Components\Plugin;
use Shopware\Components\Plugin\Context\ActivateContext;
use Shopware\Components\Plugin\Context\DeactivateContext;
use Shopware\Components\Plugin\Context\InstallContext;
use Shopware\Components\Plugin\Context\UninstallContext;
use Shopware\Components\Plugin\Context\UpdateContext;
use Symfony\Component\DependencyInjection\ContainerBuilder;

if (file_exists(__DIR__ . '/vendor/autoload.php')) {
    require_once __DIR__ . '/vendor/autoload.php';
}

class MontonioPayment extends Plugin
{
    public static $logLevel = null;
    public static $logger = null;
    public static $errorLogger = null;

    const CACHE_LIST = [
        InstallContext::CACHE_TAG_CONFIG,
        InstallContext::CACHE_TAG_HTTP,
        InstallContext::CACHE_TAG_TEMPLATE,
        InstallContext::CACHE_TAG_THEME,
        InstallContext::CACHE_TAG_ROUTER,
    ];

    public static function getSubscribedEvents()
    {
        return [
            'Enlight_Controller_Action_PreDispatch' => 'onPreDispatch',
            'Enlight_Controller_Action_PostDispatch_Frontend' => 'onPostDispatch',
        ];
    }

    public function build(
        ContainerBuilder $container
    )
    {
        parent::build($container);
        $container->addCompilerPass(new SmartyPluginCompilerPass());
    }

    public function onPostDispatch(
        Enlight_Event_EventArgs $args
    )
    {
        /** @var Enlight_Controller_Action $controller */
        $controller = $args->get('subject');
        $view = $controller->View();
        if($selectedBank = Shopware()->Session()->offsetGet('montonioBank')){
            $view->assign('montonioSelectedBank', $selectedBank);
        }
        if ($controller->Request()->get('montonio_payment_error')) {
            $this->processCheckoutError($view);
        }
    }

    protected function processCheckoutError($view)
    {
        /** @var ConfigService $configService */
        $configService = $this->container->get(ConfigService::class);
        $view->assign('sErrorMessages', [$configService->getSnippet('errorCheckout')]);
    }


    public function onPreDispatch(\Enlight_Event_EventArgs $args)
    {
        /** @var Enlight_Controller_Action $controller */
        $controller = $args->get('subject');
        $view = $controller->View();
        $view->addTemplateDir($this->getPath() . '/Resources/views');

        if($controller->Request()->get('montonioBank')){
            Shopware()->Session()->offsetSet('montonioBank', $controller->Request()->get('montonioBank'));
        }
    }

    public function activate(ActivateContext $context)
    {
        $context->scheduleClearCache(self::CACHE_LIST);
    }

    public function deactivate(DeactivateContext $context)
    {
        $context->scheduleClearCache(self::CACHE_LIST);
    }

    public function uninstall(UninstallContext $context)
    {
        if ($this->isActive()) {
            $context->scheduleClearCache(self::CACHE_LIST);
        }
    }

    public function install(InstallContext $context)
    {
        /** @var ModelManager $entityManager */
        $entityManager = $this->container->get('models');
        $paymentMethodService = new PaymentMethodService($entityManager);
        $paymentMethodService->getPaymentMethods();
    }

    public function update(UpdateContext $context)
    {
        /** @var ModelManager $entityManager */
        $entityManager = $this->container->get('models');
        $paymentMethodService = new PaymentMethodService($entityManager);
        $paymentMethodService->getPaymentMethods();
    }

}
