<?php

namespace MontonioPayment\Services;

use Exception;
use Montonio\Clients\PaymentsClient;
use Shopware\Models\Shop\DetachedShop;
use Shopware\Models\Shop\Repository;
use Shopware_Components_Config;

class ConfigService
{
    /**
     * @var Shopware_Components_Config
     */
    private $configComponent;

    public function __construct(Shopware_Components_Config $configComponent)
    {
        $this->configComponent = $configComponent;
    }

    public function getSnippet($key)
    {
        return Shopware()->Container()->get('snippets')->getNamespace('plugins/Montonio')->get($key, '', true);
    }

    public function getBackendSnippet($key)
    {
        return Shopware()->Container()->get('snippets')->getNamespace('plugins/backend/Montonio')->get($key, $key, false);
    }

    public function getValue($key, $namespace = 'MontonioPayment')
    {
        return $this->configComponent->getByNamespace($namespace, $key);
    }

    public function getAccessKey()
    {
        return $this->getValue('accessKey');
    }

    public function getSecretKey()
    {
        return $this->getValue('secretKey');
    }

    public function getEnvironment()
    {
        return $this->getValue('environment');
    }

    public function getLogLevel()
    {
        return $this->getValue('logLevel');
    }

    public function getPaymentDescription()
    {
        return str_replace(
            [
                '%d',
            ],
            [
                date('d.m.Y H:i:s'),
            ],
            $this->getValue('paymentDescription')
        );
    }

    public function getPaymentsClient(): PaymentsClient
    {
        return new PaymentsClient($this->getAccessKey(), $this->getSecretKey(), $this->getEnvironment());
    }

    public function getReturnUrl()
    {
        return $this->completeUrl(Shopware()->Front()->Router()->assemble([
            'controller' => 'MontonioPayment',
            'module' => 'Frontend',
            'action' => 'confirm',
        ]));
    }

    public function getWebhookUrl()
    {
        return $this->completeUrl(Shopware()->Front()->Router()->assemble([
            'controller' => 'MontonioPayment',
            'module' => 'Frontend',
            'action' => 'webhook',
        ]));
    }

    public function completeUrl($url)
    {
        try {
            $shop = Shopware()->Shop();
            if (empty($shop)) {
                throw new Exception();
            }
        } catch (Exception $e) {
            $modelManager = Shopware()->Container()->get('models');
            /** @var Repository $shopRepo */
            $shopRepo = $modelManager->getRepository('Shopware\Models\Shop\Shop');
            $cShop = $shopRepo->getDefault();
            $shop = DetachedShop::createFromShop($cShop);
        }

        $host = ($shop->getSecure() ? "https://" : "http://") . $shop->getHost();
        $baseUrl = rtrim($shop->getBaseUrl(), '/');

        if (strpos($url, '://') === false) {
            if (strpos($url, $baseUrl) !== 0) {
                $url = $baseUrl . $url;
            }
            $url = $host . $url;
        }

        return $url;
    }

}