<?php

namespace MontonioPayment\Services;

use Doctrine\ORM\EntityManagerInterface;
use MontonioPayment\Traits\LoggingTrait;
use Shopware\Components\Model\ModelManager;
use Shopware\Components\Plugin\PaymentInstaller;
use Shopware\Models\Dispatch\Dispatch;
use Shopware\Models\Payment\Payment;
use Shopware\Models\Plugin\Plugin;

class PaymentMethodService
{
    use LoggingTrait;
    const PAYMENT_METHOD_CODE_DIRECT_PAYMENT = 'montonio_direct_payment';
    const PAYMENT_METHOD_CODE_CARDS = 'montonio_cards';

    const MONTONIO_PAYMENT_METHOD_DIRECT_PAYMENT = 'paymentInitiation';
    const MONTONIO_PAYMENT_METHOD_CARDS = 'cardPayments';
    const PLUGIN_NAME = 'MontonioPayment';
    /**
     * @var ModelManager
     */
    protected $modelManager;

    public function __construct(EntityManagerInterface $modelManager)
    {
        $this->modelManager = $modelManager;
    }

    /**
     * @return Payment[]
     */
    public function getPaymentMethods()
    {
        $return = [];
        $paymentMethodsDetails = [
            self::PAYMENT_METHOD_CODE_DIRECT_PAYMENT => [
                'name' => self::PAYMENT_METHOD_CODE_DIRECT_PAYMENT,
                'description' => 'Montonio',
                'additionaldescription' => '',
                'action' => 'MontonioPayment',
                'active' => 1,
                'pluginID' => $this->getPluginId(),
                'position' => 1,
            ],
            self::PAYMENT_METHOD_CODE_CARDS => [
                'name' => self::PAYMENT_METHOD_CODE_CARDS,
                'description' => 'Kreditkarte',
                'additionaldescription' => '',
                'action' => 'MontonioPayment/cards',
                'active' => 1,
                'pluginID' => $this->getPluginId(),
                'position' => 1,
            ]
        ];

        foreach ($paymentMethodsDetails as $paymentMethodDetailsKey => $paymentMethodDetails) {
            /** @var Payment $payment */
            $payment = $this->modelManager->getRepository(Payment::class)->findOneBy(
                [
                    'action' => $paymentMethodDetails['action'],
                ]
            );
            if ($payment === null) {
                /** @var PaymentInstaller $installer */
                $installer = Shopware()->Container()->get('shopware.plugin_payment_installer');
                $payment = $installer->createOrUpdate($this->getPluginName(), $paymentMethodDetails);
                $this->connectShippingMethods();
            }
            $return[$paymentMethodDetailsKey] = $payment;
        }

        return $return;
    }

    public function activatePaymentMethod()
    {
        foreach($this->getPaymentMethods() as $paymentMethod){
            $paymentMethod->setActive(true);
            $this->modelManager->persist($paymentMethod);
            $this->modelManager->flush($paymentMethod);
        }
    }

    public function getPluginName()
    {
        return self::PLUGIN_NAME;
    }

    public function getPluginId()
    {

        /** @var Plugin $plugin */
        $plugin = $this->modelManager->getRepository(Plugin::class)->findOneBy(
            [
                'name' => $this->getPluginName(),
            ]
        );
        return $plugin->getId();
    }

    public function connectShippingMethods()
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            try {

                $dispatchRepository = $this->modelManager->getRepository(Dispatch::class);
                /** @var Dispatch[] $methods */
                $methods = $dispatchRepository->findAll();

                foreach ($methods as $method) {
                    foreach ($method->getPayments() as $_payment) {
                        if ($_payment->getId() === $paymentMethod->getId()) {
                            //cancel if connection already exists
                            return;
                        }
                    }
                }

                foreach ($methods as $method) {
                    foreach ($method->getPayments() as $_payment) {
                        if ($_payment->getId() === $paymentMethod->getId()) {
                            continue 2;
                        }
                    }
                    $method->getPayments()->add($paymentMethod);
                    $this->modelManager->persist($method);
                    $this->modelManager->flush($method);
                }

            } catch (\Exception $e) {
                $this->error('There has been an error connecting Montonio to your shipping methods. Please check the shipping methods manually.');
            }
        }
    }
}